@@ -53,9 +53,12 @@ class UpdateItem():
 
 
 class UpdateGroup(UpdateItem):
+    _depcache = {}
+
     def __init__(self, pkg, name, icon):
         UpdateItem.__init__(self, pkg, name, icon)
         self._items = set()
+        self._deps = set()
         self.core_item = None
         if pkg is not None:
             self.core_item = UpdateItem(pkg, name, icon)
@@ -67,41 +70,63 @@ class UpdateGroup(UpdateItem):
         all_items.extend(self._items)
         return sorted(all_items, key=lambda a: a.name.lower())
 
-    def add(self, pkg):
+    def add(self, pkg, cache=None, eventloop_callback=None):
         name = utils.get_package_label(pkg)
         icon = Gio.ThemedIcon.new("package")
         self._items.add(UpdateItem(pkg, name, icon))
+        # If the pkg is in self._deps, stop here. We have already calculated
+        # the recursive dependencies for this package, no need to do it again.
+        if cache and pkg.name in cache and pkg.name not in self._deps:
+            if not self._deps:
+                # Initial deps haven't been calculated. As we're checking
+                # whether _deps is empty in is_dependency, we must init now or
+                # it won't be done at all.
+                self._init_deps(cache, eventloop_callback)
+            self._add_deps(pkg, cache, eventloop_callback)
 
     def contains(self, item):
         return item in self._items
 
-    def _is_dependency_helper(self, cache, pkg, dep, seen, eventloop_callback):
-        if pkg is None or pkg.candidate is None or pkg in seen:
-            return False
-        elif pkg.name == dep.name:
-            return True
-        if eventloop_callback is not None:
-            eventloop_callback()
-        seen.add(pkg)
-        candidate = pkg.candidate
-        dependencies = candidate.get_dependencies('Depends', 'Recommends')
-        for dependency_pkg in itertools.chain.from_iterable(dependencies):
-            if (dependency_pkg.name in cache and
-                self._is_dependency_helper(
-                    cache, cache[dependency_pkg.name], dep, seen,
-                    eventloop_callback)):
-                return True
-        return False
-
-    def is_dependency(self, cache, maybe_dep, eventloop_callback):
-        # This is a recursive dependency check.  TODO: We do this many times
-        # when grouping packages, and it could be made more efficient.
-        seen = set()
+    def _init_deps(self, cache, eventloop_callback):
         for item in self._items:
-            if self._is_dependency_helper(cache, item.pkg, maybe_dep,
-                                          seen, eventloop_callback):
-                return True
-        return False
+            if item.pkg and item.pkg.name not in self._deps:
+                self._add_deps(item.pkg, cache, eventloop_callback)
+
+    def _add_deps(self, pkg, cache, eventloop_callback):
+        """Adds pkg and dependencies of pkg to the dependency list."""
+        if pkg is None or pkg.candidate is None or pkg.name in self._deps:
+            # This shouldn't really happen. If we land here often, it's a sign
+            # that something has gone wrong. Unless all pkgs are None it's not
+            # a critical issue - a hit to the performance at most.
+            reason = ((not pkg or not pkg.candidate)
+                      and "Package was None or didn't have a candidate."
+                      or "%s already in _deps." % pkg.name)
+            logging.debug("Useless call to _add_deps. %s" % reason)
+            return
+        if len(self._deps) % 200 == 0 and callable(eventloop_callback):
+            # Don't spin the loop every time _add_deps is called.
+            eventloop_callback()
+
+        self._deps.add(pkg.name)
+
+        if pkg.name in self._depcache:
+            for dep in self._depcache[pkg.name]:
+                if dep not in self._deps and dep in cache:
+                    self._add_deps(cache[dep], cache, eventloop_callback)
+        else:
+            candidate = pkg.candidate
+            dependencies = candidate.get_dependencies('Depends', 'Recommends')
+            for dependency_pkg in itertools.chain.from_iterable(dependencies):
+                name = dependency_pkg.name
+                if name not in self._deps and name in cache:
+                    self._depcache.setdefault(pkg.name, []).append(name)
+                    self._add_deps(cache[name], cache, eventloop_callback)
+
+    def is_dependency(self, maybe_dep, cache=None, eventloop_callback=None):
+        if not self._deps and cache:
+            self._init_deps(cache, eventloop_callback)
+
+        return maybe_dep.name in self._deps
 
     def packages_are_selected(self):
         for item in self.items:
@@ -164,7 +189,7 @@ class UpdateList():
     # the file that contains the uniq machine id
     UNIQ_MACHINE_ID_FILE = "/var/lib/dbus/machine-id"
 
-    APP_INSTALL_PATH = "/usr/share/app-install/desktop"
+    APP_INSTALL_PATTERN = "/usr/share/app-install/desktop/%s:*.desktop"
 
     # the configuration key to turn phased-updates always on
     ALWAYS_INCLUDE_PHASED_UPDATES = (
@@ -180,6 +205,7 @@ class UpdateList():
         self.security_groups = []
         self.num_updates = 0
         self.random = random.Random()
+        self.ignored_phased_updates = []
         # a stable machine uniq id
         with open(self.UNIQ_MACHINE_ID_FILE) as f:
             self.machine_uniq_id = f.read()
@@ -195,15 +221,21 @@ class UpdateList():
             self.current_desktop = os.environ.get('XDG_CURRENT_DESKTOP')
         else:
             self.current_desktop = ''
+        self.desktop_cache = {}
 
     def _file_is_application(self, file_path):
+        # WARNING: This is called often if there's a lot of updates. A poor
+        # performing call here has a huge impact on the overall performance!
+        if not file_path.endswith(".desktop"):
+            # First the obvious case: If the path doesn't end in a .desktop
+            # extension, this isn't a desktop file.
+            return False
+
         file_path = os.path.abspath(file_path)
-        is_application = False
         for app_dir in self.application_dirs:
-            is_application = is_application or file_path.startswith(app_dir)
-        extension = os.path.splitext(file_path)[1]
-        is_application = is_application and (extension == '.desktop')
-        return is_application
+            if file_path.startswith(app_dir):
+                return True
+        return False
 
     def _rate_application_for_package(self, application, pkg):
         score = 0
@@ -226,10 +258,8 @@ class UpdateList():
             if self._file_is_application(installed_file):
                 desktop_files.append(installed_file)
 
-        app_install_pattern = os.path.join(self.APP_INSTALL_PATH,
-                                           '%s:*' % pkg.name)
-        for desktop_file in glob.glob(app_install_pattern):
-            desktop_files.append(desktop_file)
+        if pkg.name in self.desktop_cache:
+            desktop_files += self.desktop_cache[pkg.name]
 
         for desktop_file in desktop_files:
             try:
@@ -250,6 +280,36 @@ class UpdateList():
         else:
             return None
 
+    def _populate_desktop_cache(self, pkg_names):
+        if not pkg_names:
+            # No updates; This shouldn't have happened.
+            logging.warning("_populate_desktop_cache called with empty list "
+                            "of packages.")
+            return
+        elif len(pkg_names) == 1:
+            # One update; Let glob do the matching.
+            pattern = self.APP_INSTALL_PATTERN % pkg_names[0]
+        else:
+            # More than one update available. Glob all desktop files and store
+            # those that match an upgradeable package.
+            pattern = self.APP_INSTALL_PATTERN % "*"
+
+        for desktop_file in glob.iglob(pattern):
+            try:
+                pkg = desktop_file.split('/')[-1].split(":")[0]
+            except IndexError:
+                # app-install-data desktop file had an unexpected naming
+                # convention. As we can't extract the package name from
+                # the path, just ignore it.
+                logging.error("Could not extract package name from '%s'. "
+                              "File ignored." % desktop_file)
+                continue
+
+            if pkg in pkg_names:
+                self.desktop_cache.setdefault(pkg, []).append(desktop_file)
+                logging.debug("App candidate for %s: %s" %
+                              (pkg, desktop_file))
+
     def _is_security_update(self, pkg):
         """ This will test if the pkg is a security update.
             This includes if there is a newer version in -updates, but also
@@ -298,13 +358,13 @@ class UpdateList():
             # multiple runs of the update-manager, so we need to
             # feed a seed that is a combination of the pkg/ver/machine
             self.random.seed("%s-%s-%s" % (
-                pkg.name, pkg.candidate.version,
+                pkg.candidate.source_name, pkg.candidate.version,
                 self.machine_uniq_id))
             threshold = pkg.candidate.record[self.PHASED_UPDATES_KEY]
             percentage = self.random.randint(0, 100)
             if percentage > int(threshold):
-                logging.info("holding back phased update (%s < %s)" % (
-                    threshold, percentage))
+                logging.info("holding back phased update %s (%s < %s)" % (
+                    pkg.name, threshold, percentage))
                 return True
         return False
 
@@ -325,54 +385,50 @@ class UpdateList():
                 'linux-image-extra-virtual']
 
     def _make_groups(self, cache, pkgs, eventloop_callback):
-        pkgs_by_source = {}
+        if not pkgs:
+            return []
         ungrouped_pkgs = []
         app_groups = []
         pkg_groups = []
 
-        # Index packages by source package name
         for pkg in pkgs:
-            srcpkg = pkg.candidate.source_name
-            pkgs_by_source.setdefault(srcpkg, []).append(pkg)
-
-        for srcpkg, pkgs in pkgs_by_source.items():
-            for pkg in pkgs:
-                app = self._get_application_for_package(pkg)
-                if app is not None:
-                    app_group = UpdateApplicationGroup(pkg, app)
-                    app_groups.append(app_group)
-                else:
-                    ungrouped_pkgs.append(pkg)
+            app = self._get_application_for_package(pkg)
+            if app is not None:
+                app_group = UpdateApplicationGroup(pkg, app)
+                app_groups.append(app_group)
+            else:
+                ungrouped_pkgs.append(pkg)
 
         # Stick together applications and their immediate dependencies
         for pkg in list(ungrouped_pkgs):
             dep_groups = []
             for group in app_groups:
-                if group.is_dependency(cache, pkg, eventloop_callback):
+                if group.is_dependency(pkg, cache, eventloop_callback):
                     dep_groups.append(group)
                     if len(dep_groups) > 1:
                         break
             if len(dep_groups) == 1:
-                dep_groups[0].add(pkg)
+                dep_groups[0].add(pkg, cache, eventloop_callback)
                 ungrouped_pkgs.remove(pkg)
 
-        # Separate out system base packages
         system_group = None
-        meta_group = UpdateGroup(None, None, None)
-        flavor_package = utils.get_ubuntu_flavor_package(cache=cache)
-        meta_pkgs = [flavor_package, "ubuntu-standard", "ubuntu-minimal"]
-        meta_pkgs.extend(self._get_linux_packages())
-        for pkg in meta_pkgs:
-            if pkg in cache:
-                meta_group.add(cache[pkg])
-        for pkg in ungrouped_pkgs:
-            if (meta_group.contains(pkg) or
-                    meta_group.is_dependency(cache, pkg, eventloop_callback)):
-                if system_group is None:
-                    system_group = UpdateSystemGroup(cache)
-                system_group.add(pkg)
-            else:
-                pkg_groups.append(UpdatePackageGroup(pkg))
+        if ungrouped_pkgs:
+            # Separate out system base packages. If we have already found an
+            # application for all updates, don't bother.
+            meta_group = UpdateGroup(None, None, None)
+            flavor_package = utils.get_ubuntu_flavor_package(cache=cache)
+            meta_pkgs = [flavor_package, "ubuntu-standard", "ubuntu-minimal"]
+            meta_pkgs.extend(self._get_linux_packages())
+            for pkg in meta_pkgs:
+                if pkg in cache:
+                    meta_group.add(cache[pkg])
+            for pkg in ungrouped_pkgs:
+                if meta_group.is_dependency(pkg, cache, eventloop_callback):
+                    if system_group is None:
+                        system_group = UpdateSystemGroup(cache)
+                    system_group.add(pkg)
+                else:
+                    pkg_groups.append(UpdatePackageGroup(pkg))
 
         app_groups.sort(key=lambda a: a.name.lower())
         pkg_groups.sort(key=lambda a: a.name.lower())
@@ -401,10 +457,14 @@ class UpdateList():
                     continue
 
                 # see if its a phased update and *not* a security update
+                # keep track of packages for which the update percentage was
+                # not met for testing
                 is_security_update = self._is_security_update(pkg)
-                if (not is_security_update and
-                        self._is_ignored_phased_update(pkg)):
-                    continue
+                if not is_security_update:
+                    if self._is_ignored_phased_update(pkg):
+                        self.ignored_phased_updates.append(pkg)
+                        pkg.mark_keep()
+                        continue
 
                 if is_security_update:
                     security_pkgs.append(pkg)
@@ -416,6 +476,10 @@ class UpdateList():
                                           pkg.marked_install):
                 self.held_back.append(pkg.name)
 
+        if security_pkgs or upgrade_pkgs:
+            # There's updates available. Initiate the desktop file cache.
+            pkg_names = [p.name for p in security_pkgs + upgrade_pkgs]
+            self._populate_desktop_cache(pkg_names)
         self.update_groups = self._make_groups(cache, upgrade_pkgs,
                                                eventloop_callback)
         self.security_groups = self._make_groups(cache, security_pkgs,
