@@ -190,7 +190,7 @@ def country_mirror():
     # special cases go here
     lang_mirror = {'c': ''}
     # no lang, no mirror
-    if not 'LANG' in os.environ:
+    if 'LANG' not in os.environ:
         return ''
     lang = os.environ['LANG'].lower()
     # check if it is a special case
@@ -212,8 +212,8 @@ def get_dist():
     # support debug overwrite
     dist = os.environ.get("META_RELEASE_FAKE_CODENAME")
     if dist:
-        logging.warn("using fake release name '%s' (because of "
-                     "META_RELEASE_FAKE_CODENAME environment) " % dist)
+        logging.warning("using fake release name '%s' (because of "
+                        "META_RELEASE_FAKE_CODENAME environment) " % dist)
         return dist
     # then check the real one
     from subprocess import Popen, PIPE
@@ -233,8 +233,8 @@ def get_dist_version():
     # support debug overwrite
     desc = os.environ.get("META_RELEASE_FAKE_VERSION")
     if desc:
-        logging.warn("using fake release version '%s' (because of "
-                     "META_RELEASE_FAKE_VERSION environment) " % desc)
+        logging.warning("using fake release version '%s' (because of "
+                        "META_RELEASE_FAKE_VERSION environment) " % desc)
         return desc
     # then check the real one
     from subprocess import Popen, PIPE
@@ -537,7 +537,7 @@ target     prot opt source               destination
         raise OSError("Need root to check the iptables state")
     if not os.path.exists("/sbin/iptables"):
         return False
-    out = subprocess.Popen(["iptables", "-L"],
+    out = subprocess.Popen(["iptables", "-nL"],
                            stdout=subprocess.PIPE,
                            universal_newlines=True).communicate()[0]
     if out == iptables_empty:
