@@ -30,6 +30,13 @@ try:
 except ImportError:
     from SocketServer import TCPServer
 
+
+from UpdateManager.Core.MetaRelease import (
+    Dist,
+    MetaReleaseCore,
+    MetaReleaseParseError,
+)
+
 CURDIR = os.path.dirname(os.path.abspath(__file__))
 
 
@@ -48,9 +55,6 @@ class SillyProxyRequestHandler(BaseHTTPRequestHandler):
     do_GET = do_HEAD
 
 
-from UpdateManager.Core.MetaRelease import MetaReleaseCore, Dist
-
-
 def get_new_dist(current_release):
     """
     common code to test new dist fetching, get the new dist information
@@ -86,7 +90,7 @@ class TestMetaReleaseCore(unittest.TestCase):
 
     def testnewdist(self):
         """ test that upgrades offer the right upgrade path """
-        for (current, next) in [("dapper", "hardy"),
+        for (current, next) in [("dapper", "edgy"),
                                 ("hardy", "lucid"),
                                 ("intrepid", "jaunty"),
                                 ("jaunty", "karmic"),
@@ -100,6 +104,12 @@ class TestMetaReleaseCore(unittest.TestCase):
     def test_url_downloadable(self):
         from UpdateManager.Core.utils import url_downloadable
         with EnvironmentVarGuard() as environ:
+            # ensure that $no_proxy doesn't prevent us from accessing
+            # localhost through proxy
+            try:
+                del environ["no_proxy"]
+            except KeyError:
+                pass
             logging.debug("proxy 1")
             environ["http_proxy"] = "http://localhost:%s/" % self.port
             install_opener(None)
@@ -158,40 +168,18 @@ class TestMetaReleaseCore(unittest.TestCase):
             self.assertTrue("<html>" in data)
 
     @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
-    def test_parse_next_release_unsupported(self, mock_download):
-        # We should jump over an unsupported release. LP: #1497024
+    def test_parse_fails_for_all_non_tagfiles(self, mock_download):
         meta = MetaReleaseCore()
-        meta.current_dist_name = "foo"
         with tempfile.TemporaryFile() as f:
-            f.write("""Dist: foo
-Supported: 1
-Date: Thu, 26 Oct 2006 12:00:00 UTC
-Version: 1.0
-
-Dist: goo
-Supported: 0
-Date: Thu, 26 Oct 2016 12:00:00 UTC
-Version: 2.0
-
-Dist: hoo
-Supported: 1
-Date: Thu, 26 Oct 2026 12:00:00 UTC
-Version: 3.0
-            """.encode("utf-8"))
+            f.write("random stuff".encode("utf-8"))
             f.seek(0)
             meta.metarelease_information = f
-            meta.parse()
-            self.assertEqual(meta.upgradable_to.name, "hoo")
-            self.assertEqual(meta.upgradable_to.version, "3.0")
-            self.assertEqual(meta.upgradable_to.supported, True)
+            self.assertRaises(MetaReleaseParseError, meta.parse)
 
     @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
-    def test_parse_next_release_unsupported_devel(self, mock_download):
-        # We should not jump over an unsupported release if we are running in
-        # "devel" mode. LP: #1497024
+    def test_parse_good(self, mock_download):
         meta = MetaReleaseCore()
         meta.current_dist_name = "foo"
-        meta.useDevelopmentRelease = True
         with tempfile.TemporaryFile() as f:
             f.write("""Dist: foo
 Supported: 1
@@ -199,7 +187,7 @@ Date: Thu, 26 Oct 2006 12:00:00 UTC
 Version: 1.0
 
 Dist: goo
-Supported: 0
+Supported: 1
 Date: Thu, 26 Oct 2016 12:00:00 UTC
 Version: 2.0
             """.encode("utf-8"))
@@ -208,7 +196,8 @@ Version: 2.0
             meta.parse()
             self.assertEqual(meta.upgradable_to.name, "goo")
             self.assertEqual(meta.upgradable_to.version, "2.0")
-            self.assertEqual(meta.upgradable_to.supported, False)
+            self.assertEqual(meta.upgradable_to.supported, True)
+
 
 if __name__ == '__main__':
     if len(sys.argv) > 1 and sys.argv[1] == "-v":
